/**
 * All the CSS types in the 'types' namespace
 */
import * as types from './types';
export { types };
/** Sets the target tag where we write the css on style updates */
export declare const setStylesTarget: (tag: {
    textContent: string;
}) => void;
/**
 * Insert `raw` CSS as a string. This is useful for e.g.
 * - third party CSS that you are customizing with template strings
 * - generating raw CSS in JavaScript
 * - reset libraries like normalize.css that you can use without loaders
 */
export declare function cssRaw(mustBeValidCSS: string): void;
/**
 * Renders styles to the singleton tag imediately
 * NOTE: You should only call it on initial render to prevent any non CSS flash.
 * After that it is kept sync using `requestAnimationFrame` and we haven't noticed any bad flashes.
 **/
export declare function forceRenderStyles(): void;
/**
 * Helps with testing. Reinitializes FreeStyle + raw
 */
export declare function reinit(): void;
/**
 * Allows use to use the stylesheet in a node.js environment
 */
export declare const getStyles: () => string;
/**
 * Takes CSSProperties and return a generated className you can use on your component
 */
export declare function style(...objects: types.NestedCSSProperties[]): string;
/**
 * Utility function to register an @font-face
 */
export declare function fontFace(...fontFace: types.FontFace[]): void;
/**
 * Takes CSSProperties and registers it to a global selector (body, html, etc.)
 */
export declare function cssRule(selector: string, ...objects: types.NestedCSSProperties[]): void;
/**
 * Takes keyframes and returns a generated animationName
 */
export declare function keyframes(frames: types.KeyFrames): string;
/**
 * Merges various styles into a single style object.
 * Note: if two objects have the same property the last one wins
 */
export declare function extend(...objects: types.NestedCSSProperties[]): types.NestedCSSProperties;
/**
 * Utility to join classes conditionally
 */
export declare function classes(...classes: (string | boolean | undefined | null)[]): string;
/**
 * Utility to help customize styles with media queries. e.g.
 * ```
 * style(
 *  media({maxWidth:500}, {color:'red'})
 * )
 * ```
 */
export declare const media: (mediaQuery: types.MediaQuery, ...objects: types.CSSProperties[]) => types.CSSProperties;
